/*
 * serial.c
 *
 * Created: 2015-09-06 00:24:35
 *  Author: DS
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"

#define BAUD_RATE		19200

//-------------------------------------------
struct {
	char wbuff[SERIAL_WBUFFER_SIZE];
	uint16_t wcounter;
	volatile char *wpt;
	
	char rbuff[SERIAL_RBUFFER_SIZE];
	uint16_t rcounter;
	volatile char rstatus;
}serial;

//-------------------------------------------
void SerialInit(void){
	UART_PORT |= (1<<UART_TXD_PIN | 1<<UART_RXD_PIN);	//config ports
	UART_DDR |= (1<<UART_TXD_PIN);
	
	//config USART
	UCSR0A = 0;
	UCSR0B = (1<<RXEN0 | 1<<TXEN0 | 1<<TXCIE0 | 1<<RXCIE0);		//RX
	UCSR0C = (1<<UCSZ01 | 1<<UCSZ00);				//8 data, 1 stop, parity none
	UBRR0H = 0;
	//UBRR0L = 25;									//UBRRL = 25 - 19.2k 8MHz
	UBRR0L = 35;									//UBRRL = 35 - 19.2k 11MHz
	
	SerialRBufferClear();
}

//-------------------------------------------
void SerialWBufferAddChar(char data){
	if (serial.wcounter < (SERIAL_WBUFFER_SIZE - 1)) {
		serial.wbuff[serial.wcounter] = data;
		serial.wcounter++;
		serial.wbuff[serial.wcounter] = 0;
	}
}

//-------------------------------------------
char SerialWBufferAdd(char *src){
	char status = 0;
	while (status == 0) {
		if (serial.wcounter < (SERIAL_WBUFFER_SIZE - 1)) {
			if ((*src) && (src)) {
				serial.wbuff[serial.wcounter] = *src;
				serial.wcounter++;
				src++;
			} else {
				status = 1;
			}
		} else status = 2;
		
		serial.wbuff[serial.wcounter] = 0;
	}
	return status;
}

//-------------------------------------------
char SerialWBufferAddLen(char *src, char len){
	char status = 0;
	while (status == 0) {
		if (serial.wcounter < (SERIAL_WBUFFER_SIZE - 1)) {
			if (len) {
				serial.wbuff[serial.wcounter] = *src;
				serial.wcounter++;
				src++;
				len--;
			} else {
				status = 1;
			}
		} else status = 2;
		
		serial.wbuff[serial.wcounter] = 0;
	}
	return status;
}

//-------------------------------------------
void SerialWBufferSend(void){
	serial.wpt = &serial.wbuff[0];
	if (*serial.wpt > 0) UDR0 = *serial.wpt;
	//while (*serial.wpt > 0){};
	/*{
		while(!(UCSR0A & 1<<UDRE0)){};
		UDR0 = *serial.wpt;
		serial.wpt++;
	}*/
}

//-------------------------------------------
void SerialWBufferNew(void){
	serial.wcounter = 0;
	serial.wbuff[0] = 0;
	serial.wpt = &serial.wbuff[0];
}

//-------------------------------------------
void SerialWBufferQuick(char *src){
	SerialWBufferNew();
	SerialWBufferAdd(src);
	SerialWBufferSend();
}

//----------------------------------------------------
ISR(USART_TX_vect){
	serial.wpt++;
	if (*serial.wpt > 0) UDR0 = *serial.wpt;
}

//-------------------------------------
void SerialSendChar(char data){
	while(!(UCSR0A & 1<<UDRE0));
	UDR0 = data;
}
//-------------------------------------------
void SerialSendMsg(char *data){
	while (*data > 0)
	{
		while(!(UCSR0A & 1<<UDRE0)){}
		UDR0 = *data;
		data++;
	}
}

//-----------------------------------
void SerialSendVal(uint16_t value){
	uint16_t temp = 10000;
	char digit;
	char blank = 1;

	while (temp)
	{
		digit = value / temp;
		value = value % temp;
		
		if (temp == 1) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank)) SerialSendChar(digit + '0');
		else SerialSendChar(' ');

		temp = temp / 10;
	}
}

//----------------------------------------------------
ISR(USART_RX_vect){
	char temp = UDR0;						//get byte from data register
	
	if (temp == COM_POSTFIX) serial.rstatus++;

	if (serial.rcounter < (SERIAL_RBUFFER_SIZE - 1))
	{
		serial.rbuff[serial.rcounter] = temp;
		serial.rcounter++;
		serial.rbuff[serial.rcounter] = 0;
	} else {
		serial.rstatus = 0xFF;	
	}
}

//----------------------------------------------------
char SerialRBufferStatus(void){
	return serial.rstatus;
}

//----------------------------------------------------
char SerialRBufferCounter(void){
	return serial.rcounter;
}

//----------------------------------------------------
void SerialRBufferClear(void){
	serial.rstatus = SERIAL_IDLE;	
	serial.rcounter = 0;
	serial.rbuff[0] = 0;
}

//----------------------------------------------------
char SerialRBufferChar(char index){
	return serial.rbuff[index];
}

//----------------------------------------------------
char *SerialRBufferPt(char index){
	return &serial.rbuff[index];
}

//------------------------------------------
int16_t SerialRBufferValue(uint16_t index){
	//char result = 0;
	char minus = 0;
	int16_t d1 = 0;
	char d2 = 0;
	char d3 = 0;

	if (serial.rbuff[index] == ' ') index++;

	//minus
	if (serial.rbuff[index] == '-')
	{
		minus = 1;
		index++;
	}

	if ((serial.rbuff[index] >= '0') && (serial.rbuff[index] <= '9'))
	{
		d1 = (serial.rbuff[index] - '0');
		index++;
		
		if ((serial.rbuff[index] >= '0') && (serial.rbuff[index] <= '9'))
		{
			d1 *= 10;
			d2 = (serial.rbuff[index] - '0');
			index++;
			
			if ((serial.rbuff[index] >= '0') && (serial.rbuff[index] <= '9'))
			{
				d1 *= 10;
				d2 *= 10;
				d3 = (serial.rbuff[index] - '0');
				index++;
			}
		}
		d1 += d2;
		d1 += d3;
		if (minus) d1 *= -1;
	}
	return d1;
}

//------------------------------------------
char SerialRBufferCharIndex(char data, char num) {
	char index = 0;
	char status = 0;
	while (status == 0) {
		if (index < SERIAL_RBUFFER_SIZE) {
			if (serial.rbuff[index] == data) num--;
			if (num == 0)  {
				status = 1;
			} else {
				index++;		
			}
		} else status = 2;
	}
	if (status == 1) return index;
	return 0;
}

//------------------------------------------
char SerialRBufferCharLastIndex(char data, char num) {
	char index = serial.rcounter;
	char status = 0;
	while (status == 0) {
		if (index <= SERIAL_RBUFFER_SIZE) {
			if (serial.rbuff[index] == data) num--;
			if (num == 0)  {
				status = 1;
			} else {
				if (index) 
					index--;
				else
					status = 2;
			}
		} else status = 2;
	}
	if (status == 1) return index;
	return 0;
}
